﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Individual {
  using System.Collections.Generic;

  using HeuristicLab.Core;
  using HeuristicLab.Encodings.IntegerVectorEncoding;
  using HeuristicLab.Optimization;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Generators.CodeGenerator;
  using HeuristicLab.Random;

  public static class IndividualExtensions {
    public static PushProgram ToPushProgram(this Individual individual, IReadOnlyPushConfiguration config, ObjectPool<IRandom> randomPool) {
      return individual.IntegerVector().ToPushProgram(config, randomPool);
    }

    public static PushProgram ToPushProgram(this Individual individual, IReadOnlyPushConfiguration config) {
      return individual.IntegerVector().ToPushProgram(config);
    }

    public static PushProgram ToPushProgram(this Individual individual, IReadOnlyPushConfiguration config, IRandom random) {
      return individual.IntegerVector().ToPushProgram(config, random);
    }

    public static PushProgram ToPushProgram(this IntegerVector vector, IReadOnlyPushConfiguration config, ObjectPool<IRandom> randomPool) {
      var random = randomPool.Allocate();
      var seed = vector.GetSeed();

      random.Reset(seed);
      var program = vector.ToPushProgram(config, random);
      randomPool.Free(random);

      return program;
    }

    public static PushProgram ToPushProgram(this IntegerVector vector, IReadOnlyPushConfiguration config) {
      var seed = (uint)vector.GetSeed();
      return vector.ToPushProgram(config, new MersenneTwister(seed));
    }

    public static int GetSeed(this IntegerVector vector) {
      var seed = 17;
      for (var i = 0; i < vector.Length; i++)
        seed = seed * 23 + vector[i];
      return seed;
    }

    public static PushProgram ToPushProgram(this IntegerVector vector, IReadOnlyPushConfiguration config, IRandom random) {
      //var currentIndex = 0;
      //var close = 0;
      var expressions = new List<Expression>(vector.Length);

      for (var i = vector.Length - 1; i >= 0; i--) {
        var expression = CodeGeneratorUtils.CreateExpressionOrErc(
          vector[i],
          random,
          config.EnabledExpressions,
          config.ErcOptions);

        expressions.Add(expression);
      }

      return new PushProgram(expressions);

      //using (var mapper = IndividualMapper.Create()) {
      //  return mapper.FromPlush(vector, ref currentIndex, ref close, 0, config, random);
      //}
    }
  }
}
